from django.urls import path
from . import views

app_name = "baykearticle"

urlpatterns = [
    path('', views.BaykeArticleContentListView.as_view(), name='article-list'),
    path('category/<int:pk>/', views.BaykeArticleCategoryDetailView.as_view(), name='category-detail'),
    path('content/<int:pk>/', views.BaykeArticleContentDetailView.as_view(), name='content-detail'),
    path('content/<int:year>/<int:month>/', views.BaykeArticleContentArchivingListView.as_view(), name='archiv-list'),
    path('content/<str:tag>/', views.BaykeArticleContentTagsListView.as_view(), name='tags-list'),
]


from django.db import models
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
# Create your models here.
from baykeshop.common.models import BaseModelMixin, ContentTypeAbstract


class BaykeClientUser(BaseModelMixin):
    """Model definition for BaykeClientUser."""
    username = models.CharField(_("终端名称"), max_length=150, blank=True, default="")
    user_agent = models.CharField(_("浏览器"), max_length=250)
    ip = models.GenericIPAddressField(_("ip地址"), protocol='both', unpack_ipv4=False)
    stats_date = models.DateField(_("统计日期"), blank=True)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeClientUser."""
        ordering = ['-add_date']
        verbose_name = _("访客")
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeClientUser."""
        return self.username
    
    @staticmethod
    def get_client_ip(request) -> str:
        x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
        if x_forwarded_for:
            return x_forwarded_for.split(',')[0]
        return request.META.get('REMOTE_ADDR')


class BaykeDataStats(ContentTypeAbstract):
    """Model definition for BaykeDataStats."""
    pv = models.PositiveIntegerField(default=0, verbose_name=_("访问量"))
    uv = models.PositiveIntegerField(default=0, verbose_name=_("访客量"))
    stats_date = models.DateField(_("统计日期"),  blank=True)

    # TODO: Define fields here

    class Meta:
        """Meta definition for BaykeDataStats."""
        ordering = ['-add_date']
        verbose_name = _("浏览量")
        verbose_name_plural = verbose_name

    def __str__(self):
        """Unicode representation of BaykeDataStats."""
        return f"uv:{self.uv}-pv:{self.pv}"

    @classmethod
    def add_stats(cls, request, content_type, object_id=None, tag=""):
        user_agent = request.headers["User-Agent"]
        ip = BaykeClientUser.get_client_ip(request)
        username = ip
        clent_user, is_created = BaykeClientUser.objects.get_or_create(
            defaults={
                "username":username,
                "ip":ip,
                "stats_date": timezone.now().date()
            },
            username = username,
            user_agent = user_agent,
            ip = ip,
            stats_date = timezone.now().date()
        )
        stats, is_stats_created = BaykeDataStats.objects.get_or_create(
            defaults={
                "content_type": content_type,
                "object_id": object_id,
                "tag": tag,
                "stats_date": timezone.now().date()
            },
            content_type=content_type,
            object_id=object_id,
            tag=tag,
            stats_date = timezone.now().date()
        )
        if is_created:
            stats.uv += 1
            stats.pv += 1
            stats.save()

        if is_stats_created:
            stats.pv += 1
            stats.save()
        
        # 5分钟之内重复同一个用户访问同一个页面只记录一次
        item_date = timezone.now() - stats.pub_date
        from datetime import timedelta
        if item_date > timedelta(minutes=5):
            stats.pv += 1
            stats.save()
        return clent_user, stats
    
    @classmethod
    def get_stats(cls, content_type, object_id=None, tag=""):
        stats = cls.objects.filter(
            models.Q(content_type=content_type)&
            models.Q(object_id=object_id)|
            models.Q(tag=tag)
        ).aggregate(models.Sum('pv'), models.Sum('uv'))
        
        pv = stats.get('pv__sum') or 1
        uv = stats.get('uv__sum') or 1
        return pv, uv



# Generated by Django 4.2.4 on 2023-10-25 13:05

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaykeClientUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('username', models.CharField(blank=True, default='', max_length=150, verbose_name='终端名称')),
                ('user_agent', models.CharField(max_length=250, verbose_name='浏览器')),
                ('ip', models.GenericIPAddressField(verbose_name='ip地址')),
                ('stats_date', models.DateField(blank=True, verbose_name='统计日期')),
            ],
            options={
                'verbose_name': '访客',
                'verbose_name_plural': '访客',
                'ordering': ['-add_date'],
            },
        ),
        migrations.CreateModel(
            name='BaykeDataStats',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('add_date', models.DateTimeField(auto_now_add=True, verbose_name='添加时间')),
                ('pub_date', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('is_delete', models.BooleanField(default=False, editable=False, verbose_name='删除标记')),
                ('object_id', models.PositiveIntegerField(blank=True, null=True)),
                ('tag', models.CharField(blank=True, default='', max_length=200, verbose_name='标记')),
                ('pv', models.PositiveIntegerField(default=0, verbose_name='访问量')),
                ('uv', models.PositiveIntegerField(default=0, verbose_name='访客量')),
                ('stats_date', models.DateField(blank=True, verbose_name='统计日期')),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
            options={
                'verbose_name': '浏览量',
                'verbose_name_plural': '浏览量',
                'ordering': ['-add_date'],
            },
        ),
    ]



from rest_framework import permissions

class IsOwnerAuthenticated(permissions.IsAuthenticated):
    """ 仅拥有获取自己个人相关信息的权限 """
    
    def has_permission(self, request, view):
        return super().has_permission(request, view)
    
    def has_object_permission(self, request, view, obj):
        return bool(request.user == obj.owner)


from rest_framework import renderers
from rest_framework.utils.serializer_helpers import ReturnList
from django.core.serializers.json import DjangoJSONEncoder


class JSONRenderer(renderers.JSONRenderer):
    
    def render(self, data, accepted_media_type=None, renderer_context=None):
        return super().render(data, accepted_media_type, renderer_context)
    

class TemplateHTMLRenderer(renderers.TemplateHTMLRenderer):
    
    def render(self, data, accepted_media_type=None, renderer_context=None):
        if isinstance(data, ReturnList):
            data = {"results": data}
        return super().render(data, accepted_media_type, renderer_context)


from typing import Any, Dict
from django.db.models import Q
from django.http import HttpRequest, HttpResponse
from django.views.generic import ListView, DetailView
from django.views.generic.detail import SingleObjectMixin
from django.contrib.contenttypes.models import ContentType
# Create your views here.
from baykeshop.apps.stats.models import BaykeDataStats
from .models import BaykeArticleContent, BaykeArticleCategory


class BaykeArticleContentListView(ListView):
    """列表页

    Args:
        ListView (_type_): _description_

    Returns:
        _type_: _description_
    """
    model = BaykeArticleContent
    template_name = "article/list.html"
    paginate_by = 20
    paginate_orphans = 2
    context_object_name = "article_list"

    def get_queryset(self):
        queryset = super().get_queryset().exclude(status=0)
        content_type = ContentType.objects.get_for_model(BaykeArticleContent)
        for qs in queryset:
            qs.pv, qs.uv = BaykeDataStats.get_stats(content_type=content_type, object_id=qs.id)
        return queryset
    
    def paginate_queryset(self, queryset, page_size):
        page_size = int(self.request.GET.get('per_page', self.paginate_by))
        return super().paginate_queryset(queryset, page_size)

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = "全部文章"
        return context
    
    def add_stats(self):
        content_type = ContentType.objects.get_for_model(self.model)
        clent_user, stats = BaykeDataStats.add_stats(
            request=self.request, content_type=content_type, 
            tag=self.request.path_info
        )
        return stats


class BaykeArticleCategoryDetailView(SingleObjectMixin, BaykeArticleContentListView):
    """分类列表页

    Args:
        SingleObjectMixin (_type_): _description_
        BaykeArticleContentListView (_type_): _description_
    """
    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=BaykeArticleCategory.objects.all())
        self.add_stats()
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["category"] = self.object
        context['title'] = self.object.name
        return context

    def get_queryset(self):
        queryset = self.object.baykearticlecontent_set.filter(status=1)
        content_type = ContentType.objects.get_for_model(BaykeArticleContent)
        for qs in queryset:
            qs.pv, qs.uv = BaykeDataStats.get_stats(content_type=content_type, object_id=qs.id)
        return queryset
    
    def add_stats(self):
        content_type = ContentType.objects.get_for_model(BaykeArticleCategory)
        clent_user, stats = BaykeDataStats.add_stats(
            self.request, content_type, 
            self.object.id, self.request.path_info
        )
        return stats
    
    def get_stats(self):
        content_type = ContentType.objects.get_for_model(BaykeArticleCategory)
        pv, uv = BaykeDataStats.get_stats(
            content_type=content_type, 
            object_id=self.object.id, 
            tag=self.request.path_info
        )
        return pv, uv

class BaykeArticleContentDetailView(DetailView):
    """文章详情页

    Args:
        DetailView (_type_): _description_
    """
    model = BaykeArticleContent
    template_name = "article/detail.html"
    context_object_name = "article"

    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['article_next'] = self.get_object().next_article
        context['article_previous'] = self.get_object().previous_article
        context['title'] = self.get_object().title
        context['pv'], context['uv'] = self.get_stats()
        return context
    
    def get(self, request: HttpRequest, *args: Any, **kwargs: Any) -> HttpResponse:
        self.add_stats()
        return super().get(request, *args, **kwargs)

    def add_stats(self):
        # 新增当天统计数据
        content_type = ContentType.objects.get_for_model(self.model)
        clent_user, stats = BaykeDataStats.add_stats(
            self.request, content_type, 
            self.get_object().id, self.request.path_info
        )
        return stats
    
    def get_stats(self):
        # 获取统计数据
        content_type = ContentType.objects.get_for_model(self.model)
        pv, uv = BaykeDataStats.get_stats(
            content_type=content_type, 
            object_id=self.get_object().id, 
            tag=self.request.path_info
        )
        return pv, uv


class BaykeArticleContentArchivingListView(BaykeArticleContentListView):
    """ 文章归档 """
    
    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = f"{self.kwargs['year']}年{self.kwargs['month']}月的文章归档"
        return context

    def get_queryset(self):
        queryset = super().get_queryset().filter(
            Q(add_date__year=self.kwargs['year']) &
            Q(add_date__month=self.kwargs['month'])
        )
        content_type = ContentType.objects.get_for_model(BaykeArticleContent)
        for qs in queryset:
            qs.pv, qs.uv = BaykeDataStats.get_stats(content_type=content_type, object_id=qs.id)
        return queryset
    

class BaykeArticleContentTagsListView(BaykeArticleContentListView):
    """ 文章标签分类 """
    
    def get_context_data(self, **kwargs: Any) -> Dict[str, Any]:
        context = super().get_context_data(**kwargs)
        context['title'] = f"归属标签{self.kwargs['tag']}的文章"
        return context

    def get_queryset(self):
        queryset = super().get_queryset().filter(tags__name=self.kwargs['tag'])
        content_type = ContentType.objects.get_for_model(BaykeArticleContent)
        for qs in queryset:
            qs.pv, qs.uv = BaykeDataStats.get_stats(content_type=content_type, object_id=qs.id)
        return queryset


